//////////////////////////////////////////////////////////////////////////////////////
//
//  Copyright (c) 2014-present, Egret Technology.
//  All rights reserved.
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions are met:
//
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the Egret nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY EGRET AND CONTRIBUTORS "AS IS" AND ANY EXPRESS
//  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
//  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
//  IN NO EVENT SHALL EGRET AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;LOSS OF USE, DATA,
//  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
//  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
//////////////////////////////////////////////////////////////////////////////////////

class Main extends egret.DisplayObjectContainer {

    /**
     * 加载进度界面
     * Process interface loading
     */
    private loadingView: LoadingUI;
    private car1: egret.Bitmap;
    private car2: egret.Bitmap;
    private start: egret.Bitmap;
    private pause: egret.Bitmap;
    private pause_start: egret.Bitmap;
    private stop: egret.Bitmap;
    private topMask: egret.Shape;
    private line: egret.Shape;
    private icon: egret.Bitmap;
    private colorLabel: egret.TextField;
    private textfield: egret.TextField;
    private left_mm: number;
    private right_mm: number;
    private run_left_1: egret.Bitmap;
    private run_left_2: egret.Bitmap;
    private run_right_1: egret.Bitmap;
    private run_right_2: egret.Bitmap;
    private speed: number;
    private isPause: boolean;

    public constructor() {
        super();
        this.addEventListener(egret.Event.ADDED_TO_STAGE, this.onAddToStage, this);
    }

    private onAddToStage(event: egret.Event) {
        //设置加载进度界面
        //Config to load process interface
        this.loadingView = new LoadingUI();
        this.stage.addChild(this.loadingView);

        //初始化Resource资源加载库
        //initiate Resource loading library
        RES.addEventListener(RES.ResourceEvent.CONFIG_COMPLETE, this.onConfigComplete, this);
        RES.loadConfig("resource/default.res.json", "resource/");
    }

    /**
     * 配置文件加载完成,开始预加载preload资源组。
     * configuration file loading is completed, start to pre-load the preload resource group
     */
    private onConfigComplete(event: RES.ResourceEvent): void {
        RES.removeEventListener(RES.ResourceEvent.CONFIG_COMPLETE, this.onConfigComplete, this);
        RES.addEventListener(RES.ResourceEvent.GROUP_COMPLETE, this.onResourceLoadComplete, this);
        RES.addEventListener(RES.ResourceEvent.GROUP_LOAD_ERROR, this.onResourceLoadError, this);
        RES.addEventListener(RES.ResourceEvent.GROUP_PROGRESS, this.onResourceProgress, this);
        RES.addEventListener(RES.ResourceEvent.ITEM_LOAD_ERROR, this.onItemLoadError, this);
        RES.loadGroup("preload");
    }

    /**
     * preload资源组加载完成
     * Preload resource group is loaded
     */
    private onResourceLoadComplete(event: RES.ResourceEvent) {
        if (event.groupName == "preload") {
            this.stage.removeChild(this.loadingView);
            RES.removeEventListener(RES.ResourceEvent.GROUP_COMPLETE, this.onResourceLoadComplete, this);
            RES.removeEventListener(RES.ResourceEvent.GROUP_LOAD_ERROR, this.onResourceLoadError, this);
            RES.removeEventListener(RES.ResourceEvent.GROUP_PROGRESS, this.onResourceProgress, this);
            RES.removeEventListener(RES.ResourceEvent.ITEM_LOAD_ERROR, this.onItemLoadError, this);
            this.createGameScene();
        }
    }

    /**
     * 资源组加载出错
     *  The resource group loading failed
     */
    private onItemLoadError(event: RES.ResourceEvent) {
        console.warn("Url:" + event.resItem.url + " has failed to load");
    }

    /**
     * 资源组加载出错
     *  The resource group loading failed
     */
    private onResourceLoadError(event: RES.ResourceEvent) {
        //TODO
        console.warn("Group:" + event.groupName + " has failed to load");
        //忽略加载失败的项目
        //Ignore the loading failed projects
        this.onResourceLoadComplete(event);
    }

    /**
     * preload资源组加载进度
     * Loading process of preload resource group
     */
    private onResourceProgress(event: RES.ResourceEvent) {
        if (event.groupName == "preload") {
            this.loadingView.setProgress(event.itemsLoaded, event.itemsTotal);
        }
    }

    /**
     * 创建游戏场景
     * Create a game scene
     */
    private createGameScene() {
        let sky = this.createBitmapByName("bg_png");
        this.addChild(sky);
        let stageW = this.stage.stageWidth;
        let stageH = this.stage.stageHeight;
        sky.width = stageW;
        sky.height = stageH;

        this.showMessage();

        //增加两辆车
        let car1 = this.createBitmapByName("car0_png");
        this.addChild(car1);
        car1.width = 40;
        car1.height = 80;
        car1.x = 90;
        car1.y = 400;
        this.car1 = car1;
        let car2 = this.createBitmapByName("car2_png");
        this.addChild(car2);
        car2.width = 40;
        car2.height = 80;
        car2.x = 240;
        car2.y = 400;
        this.car2 = car2;

        //增加开始按钮
        let start = this.createBitmapByName("startGameBtn_png");
        start.width = 160;
        start.height = 50;
        start.x = 100;
        start.y = 350;
        start.touchEnabled = true; //设置可以进行触摸
        this.start = start;
        this.addChild(start);


        let run_left_1 = this.createBitmapByName("runleft_png");
        let run_left_2 = this.createBitmapByName("runleft_png");
        let run_right_1 = this.createBitmapByName("runright_png");
        let run_right_2 = this.createBitmapByName("runright_png");
        this.run_left_1 = run_left_1;
        this.run_left_2 = run_left_2;
        this.run_right_1 = run_right_1;
        this.run_right_2 = run_right_2;
        run_left_1.width = 80;
        run_left_1.height = 480;
        run_left_2.width = 80;
        run_left_2.height = 480;
        run_left_1.x = 0;
        run_left_1.y = 0;
        run_left_2.x = 0;
        run_left_2.y = 480;
        run_right_1.width = 80;
        run_right_1.height = 480;
        run_right_2.width = 80;
        run_right_2.height = 480;
        run_right_1.x = 280;
        run_right_1.y = 0;
        run_right_2.x = 280;
        run_right_2.y = 480;
        this.addChild(this.run_left_1);
        this.addChild(this.run_left_2);
        this.addChild(this.run_right_1);
        this.addChild(this.run_right_2);
        this.run_left_1.visible = false;
        this.run_left_2.visible = false;
        this.run_right_1.visible = false;
        this.run_right_2.visible = false;

        //增加停止 暂停按钮
        //放在跑道上面
        let pause = this.createBitmapByName("pause_png");
        pause.width = 50;
        pause.height = 50;
        pause.x = 250;
        pause.y = 10;
        pause.touchEnabled = true; //设置可以进行触摸
        pause.visible = false;
        this.pause = pause;
        this.addChild(pause);
        let pause_start = this.createBitmapByName("start_png");
        pause_start.width = 50;
        pause_start.height = 50;
        pause_start.x = 250;
        pause_start.y = 10;
        pause_start.touchEnabled = true; //设置可以进行触摸
        pause_start.visible = false;
        this.pause_start = pause_start;
        this.addChild(pause_start);
        let stop = this.createBitmapByName("stop_png");
        stop.width = 50;
        stop.height = 50;
        stop.x = 300;
        stop.y = 10;
        stop.touchEnabled = true; //设置可以进行触摸
        stop.visible = false;
        this.stop = stop;
        this.addChild(stop);

        this.isPause = true;

        //初始化两个小车移动路程为0
        this.left_mm = 0;
        this.right_mm = 0;

        this.addEventListener(egret.TouchEvent.TOUCH_TAP, this.onClickStart, this);
        RES.getResAsync("description_json", this.startAnimation, this);
    }

    /**
     * 根据name关键字创建一个Bitmap对象。name属性请参考resources/resource.json配置文件的内容。
     * Create a Bitmap object according to name keyword.As for the property of name please refer to the configuration file of resources/resource.json.
     */
    private createBitmapByName(name: string) {
        let result = new egret.Bitmap();
        let texture: egret.Texture = RES.getRes(name);
        result.texture = texture;
        return result;
    }

    /**
     * 描述文件加载成功，开始播放动画
     * Description file loading is successful, start to play the animation
     */
    private startAnimation(result: string[]) {
        let parser = new egret.HtmlTextParser();

        let textflowArr = result.map(text => parser.parse(text));
        let textfield = this.textfield;
        let count = -1;
        let change = () => {
            count++;
            if (count >= textflowArr.length) {
                count = 0;
            }
            let textFlow = textflowArr[count];

            // 切换描述内容
            // Switch to described content
            textfield.textFlow = textFlow;
            let tw = egret.Tween.get(textfield);
            tw.to({ "alpha": 1 }, 200);
            tw.wait(2000);
            tw.to({ "alpha": 0 }, 200);
            tw.call(change, this);
        };

        change();
    }

    /**
     * 定义公司产品信息
     */
    private showMessage(){
        let topMask = new egret.Shape();
        topMask.graphics.beginFill(0xFF0000, 0.5);
        topMask.graphics.drawRect(0, 0, this.stage.stageWidth, 172);
        topMask.graphics.endFill();
        topMask.y = 33;
        this.addChild(topMask);
        this.topMask = topMask;

        let icon = this.createBitmapByName("egret_icon_png");
        this.addChild(icon);
        icon.x = 26;
        icon.y = 33;
        this.icon = icon;

        let line = new egret.Shape();
        line.graphics.lineStyle(2, 0xffffff);
        line.graphics.moveTo(0, 0);
        line.graphics.lineTo(0, 117);
        line.graphics.endFill();
        line.x = 172;
        line.y = 61;
        this.addChild(line);
        this.line = line;

        let colorLabel = new egret.TextField();
        colorLabel.textColor = 0xffffff;
        colorLabel.width = this.stage.stageWidth - 172;
        colorLabel.textAlign = "center";
        colorLabel.text = "JL 蓝牙运动手环";
        colorLabel.size = 24;
        colorLabel.x = 172;
        colorLabel.y = 80;
        this.addChild(colorLabel);
        this.colorLabel = colorLabel;

        let textfield = new egret.TextField();
        this.addChild(textfield);
        textfield.alpha = 0;
        textfield.width = this.stage.stageWidth - 172;
        textfield.textAlign = egret.HorizontalAlign.CENTER;
        textfield.size = 24;
        textfield.textColor = 0xffffff;
        textfield.x = 172;
        textfield.y = 135;
        this.textfield = textfield;
    }
    /**
     * 自定义一些事件 // 所有的触摸事件都会在这里进行捕获
     */
    private onClickStart(event: egret.Event) {
        //debugger;
        //console.log("click start" + event);
        if(event.target == this.start){
            //删除提示信息
            this.removeChild(this.topMask);
            this.removeChild(this.icon);
            this.removeChild(this.line);
            this.removeChild(this.colorLabel);
            this.removeChild(this.textfield);
            this.removeChild(this.start);

            this.run_left_1.visible = true;
            this.run_left_2.visible = true;
            this.run_right_1.visible = true;
            this.run_right_2.visible = true;
            this.pause.visible = true;
            this.stop.visible = true;
            this.speed = 5;
            this.isPause = false;

            this.removeEventListener(egret.Event.ENTER_FRAME, this.enterFrameHandler, this);
            this.addEventListener(egret.Event.ENTER_FRAME, this.enterFrameHandler, this);
        }else if(event.target == this.stop){
            console.log("stop");
            this.onClickStop();
        }else if(event.target == this.pause ||event.target == this.pause_start){
            console.log("pause");
            this.onClickPause();
        }
        //this.backGroud.start();
        //正中间显示时间
        //左右两边显示距离

        //left 接受事件，并处理小车运动
        //right 接受事件，并处理小车运动

        //通过websocket发送接收数据，然后画动画
        //http://www.cnblogs.com/yangxiao/p/4686729.html
    }

	/**
	 * 逐帧运动
	 */
	public enterFrameHandler(event: egret.Event) : void{
        var y = this.run_left_1.y;
        if(y >= 480){
            y = -460;
        }else{
            y = y + this.speed;
        }
        this.run_left_1.y = y;
        this.run_right_1.y = y;
        y = this.run_left_2.y;
        if(y >= 480){
            y = -460;
        }else{
            y = y + this.speed;
        }
        this.run_left_2.y = y;
        this.run_right_2.y = y;
        
	}
	/**
	 * 停止
	 */
	public onClickStop(): void{
		this.removeEventListener(egret.Event.ENTER_FRAME, this.enterFrameHandler, this);
        this.isPause = true;
        //this.speed = 0;
	}
    /**
     * 暂停
     */
    public onClickPause(): void{
        if(this.isPause == true){
            this.isPause = false;
            this.pause.visible = true;
            this.pause_start.visible = false;
            this.addEventListener(egret.Event.ENTER_FRAME, this.enterFrameHandler, this);
        }else{
            this.isPause = true;
            this.pause.visible = false;
            this.pause_start.visible = true;
            this.removeEventListener(egret.Event.ENTER_FRAME, this.enterFrameHandler, this);
        }
    }
}


